@@ -35,7 +35,7 @@ checkCluster <- function(cl) {
 #
 
 slaveLoop <- function(master) {
-    repeat {
+    repeat tryCatch({
         msg <- recvData(master)
 	cat(paste("Type:", msg$type, "\n"))
 
@@ -61,7 +61,7 @@ slaveLoop <- function(master) {
                           time = t2 - t1, tag = msg$data$tag)
             sendData(master, value)
         }
-    }
+    }, interrupt = function(e) NULL)
 }
 
 sinkWorkerOutput <- function(outfile) {
@@ -174,12 +174,28 @@ stopCluster.default <- function(cl)
 # Cluster Functions
 #
 
-sendCall <- function(con, fun, args, return = TRUE, tag = NULL)
+sendCall <- function (con, fun, args, return = TRUE, tag = NULL) {
     #**** mark node as in-call
-    postNode(con, "EXEC", list(fun = fun, args = args, return = return,
+    timing <-  .snowTimingData$running()
+    if (timing)
+        start <- proc.time()[3]
+    postNode(con, "EXEC", list(fun = fun, args = args, return = return, 
                                tag = tag))
+    if (timing)
+        .snowTimingData$enterSend(con$rank, start, proc.time()[3])
+    NULL
+}
 
-recvResult <- function(con) recvData(con)$value
+recvResult <- function (con)  {
+  if (.snowTimingData$running()) {
+      start <- proc.time()[3]
+      r <- recvData(con)
+      end <- proc.time()[3]
+      .snowTimingData$enterRecv(con$rank, start, end, r$time[3])
+  }
+  else r <- recvData(con)
+  r$value
+}
 
 checkForRemoteErrors <- function(val) {
     count <- 0
@@ -253,8 +269,14 @@ clusterExport <- local({
 #     }
 # }
 
-recvOneResult <- function(cl) {
-    v <- recvOneData(cl)
+recvOneResult <- function (cl) {
+    if (.snowTimingData$running()) {
+        start <- proc.time()[3]
+        v <- recvOneData(cl)
+        end <- proc.time()[3]
+        .snowTimingData$enterRecv(v$node, start, end, v$value$time[3])
+    }
+    else v <- recvOneData(cl)
     list(value = v$value$value, node = v$node, tag = v$value$tag)
 }
 
